@@ -22,6 +22,7 @@ from kombu.utils.objects import cached_property
 from kombu.utils.scheduling import cycle_by_name
 from kombu.utils.url import _parse_url
 from kombu.utils.uuid import uuid
+from kombu.utils.compat import _detect_environment
 
 from . import virtual
 
@@ -196,6 +197,9 @@ class QoS(virtual.QoS):
             try:
                 with Mutex(client, self.unacked_mutex_key,
                            self.unacked_mutex_expire):
+                    env = _detect_environment()
+                    if env == 'gevent':
+                        ceil = time()
                     visible = client.zrevrangebyscore(
                         self.unacked_index_key, ceil, 0,
                         start=num and start, num=num, withscores=True)
@@ -927,8 +931,22 @@ class Channel(virtual.Channel):
         if asynchronous:
             class Connection(connection_cls):
                 def disconnect(self):
-                    super(Connection, self).disconnect()
+                    # NOTE: see celery issue #3898
+                    # redis-py Connection shutdown()s the socket
+                    # which causes all copies of file descriptor
+                    # to become unusable, however close() only
+                    # affect process-local copies of fds.
+                    # So we just override Connection's disconnect method.
+                    self._parser.on_disconnect()
                     channel._on_connection_disconnect(self)
+                    if self._sock is None:
+                        return
+                    try:
+                        # self._sock.shutdown(socket.SHUT_RDWR)
+                        self._sock.close()
+                    except socket.error:
+                        pass
+                    self._sock = None
             connection_cls = Connection
 
         connparams['connection_class'] = connection_cls
@@ -1058,9 +1076,7 @@ class SentinelChannel(Channel):
 
     sentinel://0.0.0.0:26379;sentinel://0.0.0.0:26380/...
 
-    where each sentinel is separated by a `;`.  Multiple sentinels are handled
-    by :class:`kombu.Connection` constructor, and placed in the alternative
-    list of servers to connect to in case of connection failure.
+    where each sentinel is separated by a `;`.
 
     Other arguments for the sentinel should come from the transport options
     (see :method:`Celery.connection` which is in charge of creating the
@@ -1084,11 +1100,14 @@ class SentinelChannel(Channel):
 
         additional_params.pop('host', None)
         additional_params.pop('port', None)
-
+        connection_list = []
+        for url in self.connection.client.alt:
+            if url and 'sentinel://' in url:
+                connection_list.append(url.split('/')[2].split(':'))
         sentinel_inst = sentinel.Sentinel(
-            [(connparams['host'], connparams['port'])],
+            connection_list,
             min_other_sentinels=getattr(self, 'min_other_sentinels', 0),
-            sentinel_kwargs=getattr(self, 'sentinel_kwargs', {}),
+            sentinel_kwargs=getattr(self, 'sentinel_kwargs', None),
             **additional_params)
 
         master_name = getattr(self, 'master_name', None)
